% MATLAB function for generating sample data matrices by using Hong's (1999) method.
% Based on: Hong, S. (1999). Generating correlation matrices with model
% error for simulation studies in factor analysis: A combination of
% the Tucker-Koopman-Linn model and Wijsmans algorithm. Behavior
% Research Methods, Instruments, & Computers, 31, 727730.
function [OBS, F]=Hong(r,k,n,V,mfv,ifc,eps)
% ifc=interfactor correlation
% r common (major) factors
% k measured variables
% n sample size

q=200;  %q minor factors
%V=   k*r             TrueLoadings / major factor loading matrix
%D=   k*k             Diagonal matrix of unique factor variances
%W=   k*q             Minor factor loading matrix
%J=   k*(r+q)         Super loading matrix with columns made up of V and W
%B=   (r+q) * (r+q)   Super matrix of factor correlations
%S=   r*r             Correlations among major factors
%H=   r*q             Correlations among major and minor factors
%L=   q*q             Correlations among minor factors
%P=   k*k             Population correlation matrix of order k
%X=   n*k             Random matrix with mean 0 and variance 1
%F=   k*k             Principal component matrix of P
%U=   k*k             Matrix with columns  from the k eigenvectors of P
%D=   k*k             Diagonal matrix with the k eigenvalues as elements
%OBS= n*k             Multivariate normal population with 0 means and 1 SDs

S=diag(1*ones(r,1));S(S==0)=ifc; % diagonal elements=1; off-diagonal elements=ifc
H=ifc*ones(r,q); % all elements=0.3
L=diag(ones(q,1)); % diagonal elements=1; off-diagonal elements=ifc

B=[S H;H' L];

W=zeros(k,q);
if mfv>0   % if model error
    SD=1;
    for i=1:q
        W(1:k,i)=randn(1,k).*SD;
        SD=SD*(1-eps);
    end
    for i=1:k
        W(i,:)=W(i,:)./  sqrt((1/mfv*sum(W(i,:).^2)));
    end
end
J=[V W];
JBJ=J*B*J';
D=eye(k)-diag(diag(JBJ));
P=JBJ+D;
[U,D2]=eig(P);
F=U*D2^.5;
X=randn(n,k);
OBS=X*F';