% MATLAB code for the plasmode study (Thurstones box problem) presented in the paper:
% De Winter, J. C. F., & Dodou, D. (2012). Factor recovery by principal axis factoring and maximum likelihood factor analysis as a function of factor pattern and sample size. Journal of Applied Statistics, 39, 695-710.
% This code was tested with Matlab versions R2011b and R2012b, and re-tested with Matlab R0218b

close all;clear all;clc;warning off all %#ok<CLALL>
nfactors=3;tol=10^-3;itermax=9999;

X=NaN(10^7,23);
X(:,1:3)=1+9*rand(size(X,1),3); % uniform x, y, and z between 1 and 9
X=appendbox(X);

r=corrcoef(X);
maxlike;

TL=[1 0 0;0 1 0; 0 0 1;zeros(size(X,2)-3,nfactors)];
[~,T2]=rotatefactors(lding(1:nfactors,:),'method','procrustes','target',[1 0 0;0 1 0;0 0 1],'type','oblique');
lding=lding*T2;
TL=ordering(lding,TL);

disp('Table 7. True loading coefficients of the Thurstones box problem')
disp(round(TL,3))
%%
reps=2500;
[G1,G2]=deal(NaN(reps,1));

for iii=1:reps
    X=NaN(25,23);
    X(:,1:3)=1+9*rand(25,3); % uniform x, y, and z between 1 and 9
    X(:,2)=X(:,1)*2+X(:,2); % this line introduces a strong correlation between x and y. Remove this line if wanting to keep x and y uncorrelated
    
    X=appendbox(X);
    r=corrcoef(X);
    
    L_PAF=paf(r,nfactors,tol,itermax);
    L_PAF=FArot(L_PAF,3,TL);
    [L_PAF,TL_PAFordered]=ordering(L_PAF,TL);
    
    maxlike;
    L_MLFA=FArot(lding,3,TL);
    [L_MLFA,TL_MLFAordered]=ordering(L_MLFA,TL);
    
    if max(max(L_MLFA))<1.5 && max(max(L_PAF))<1.5 % only if mlfa is feasible; impossible outcomes are discarded
        G1(iii)=sqrt(mean(mean((L_PAF-TL_PAFordered).^2)));
        G2(iii)=sqrt(mean(mean((L_MLFA-TL_MLFAordered).^2)));
    end
end
%%
figure;plot(G1,G2,'o');hold on;plot([0 1],[0 1],'k')
xlabel('{\itg} PAF')
ylabel('{\itg} MLFA')
disp(cohend(G2,G1))
disp(sum(G1>G2)/sum(~isnan(G1)))
%%
function X=appendbox(X)
X(:,4)=X(:,1).^2;
X(:,5)=X(:,2).^2;
X(:,6)=X(:,3).^2;
X(:,7)=X(:,1).*X(:,2);
X(:,8)=X(:,1).*X(:,3);
X(:,9)=X(:,2).*X(:,3);
X(:,10)=(X(:,1).^2+X(:,2).^2).^.5;
X(:,11)=(X(:,1).^2+X(:,3).^2).^.5;
X(:,12)=(X(:,2).^2+X(:,3).^2).^.5;
X(:,13)=2*X(:,1)+2*X(:,2);
X(:,14)=2*X(:,1)+2*X(:,3);
X(:,15)=2*X(:,2)+2*X(:,3);
X(:,16)=log10(X(:,1));
X(:,17)=log10(X(:,2));
X(:,18)=log10(X(:,3));
X(:,19)=X(:,1).*X(:,2).*X(:,3);
X(:,20)=(X(:,1).^2+X(:,2).^2+X(:,3).^2).^.5;
X(:,21)=exp(X(:,1));
X(:,22)=exp(X(:,2));
X(:,23)=exp(X(:,3));
end