% MATLAB function for recovering the order and sign of the loadings.
function [Lordered,TLreduced]=ordering(L,TL)    % input is are the loadings and the true loadings
m=size(TL,2);        % number of factors of TL
m2=size(L,2);        % number of factors of L
order=NaN(m,1);
sgn=NaN(m,1);
G=NaN(m,m2);

% Congruence coefficient (K) between all combinations of dimensions
for i=1:m
    for i2=1:m2
        G(i,i2)=cc(TL(:,i),L(:,i2));
    end
end
temp=sort(reshape(abs(G),1,m*m2),'descend'); % sort the absolute of all G values in descending order. temp is a m*m2 vector
for i=1:m*m2  % loop across all possible combinations of dimensions of loadings and true loadings
    [O,F]=find(abs(G)==temp(i)); % find the i-th smallest G value from the G matrix (O=row number out of m; F=column number out of m2)
    O=O(1);F=F(1); % take the first one, if there happen to be identical Gs
    if isnan(order(O)) && ~ismember(F,order)  % if dimension O of TL is not yet used, and if dimension F of L is not yet used
        order(O)=F; % specify that dimension O of TL belongs to dimension F of L
        sgn(O)=sign(G(O,F)); % store the sign of the G value
        G(O,F)=NaN; % erase the G value from the G matrix, so it will not be found again
    end
end
dims=find(order>0); % dims specifies which columns of TL were used
order=order(dims);
sgn=sgn(dims);
Lordered=L(:,order);  % the ordered pattern matrix. Lordered is a p x m3 matrix, with m3=min([m m2])
Lordered=Lordered.*repmat(sgn',size(Lordered,1),1); % recover sign of Lordered
TLreduced=TL(:,dims);
